'use strict';
/**
 * Module dependencies.
 */
var Request = require('./../../lib/request');
var Utils = require('./../../lib/utils');

/**
 * Helper for expect and only
 * @api private
 */
var _extend = function (type) {
    return function () {
        this._query[type] = this._query[type] || {};
        switch (arguments.length) {
            case 1:
                if (Array.isArray(arguments[0]) || typeof arguments[0] === "string") {
                    var query = this._query[type]['BASE'] || [];
                    if (typeof arguments[0] === "string") {
                        query.push(arguments[0]);
                    } else {
                        query = query.concat(arguments[0]);
                    }
                    this._query[type]['BASE'] = query;
                    return this;
                } else {
                    console.error("Kindly provide valid parameters");
                }
                break;
            case 2:
                if (typeof arguments[0] === "string" && (Array.isArray(arguments[1]) || typeof arguments[1] === "string")) {
                    var query = this._query[type][arguments[0]] || [];
                    if (typeof arguments[1] === "string") {
                        query.push(arguments[1]);
                    } else {
                        query = query.concat(arguments[1]);
                    }
                    this._query[type][arguments[0]] = query;
                    return this;
                } else {
                    console.error("Kindly provide valid parameters");
                }
                break;
            default:
                console.error("Kindly provide valid parameters");
        }
    };
};

function Entry() {
    this._query = {};
    return this;
}

/**
 * @method only
 * @description Entry to filter entries.
 * @param {String} key - find key in entry
 * @param {Array} values - match or compare value in entry
 * @returns {Entry}
 * @example .only('title') || .only('BASE','title') ||  .only(['title','description']) || .only('refernce_field_uid','title')
 */
Entry.prototype.only = _extend('only');

/**
 * @method except
 * @description Entry to filter entries.
 * @param {String} key - find key in entry
 * @param {Array} values - match or compare value in entry
 * @returns {Entry}
 * @example .except('title') || .except('BASE','title') ||  .except(['title','description']) || .except('refernce_field_uid','title')
 */
Entry.prototype.except = _extend('except');

/**
 * @method include
 * @description Include reference entries response.
 * @returns {Entry}
 * @example .include(['field_uid1','field_uid2']) || .include('field_uid1','field_uid2')
 */
Entry.prototype.include = function (val) {
    if (Array.isArray(val) || typeof val === "string") {
        this._query['include'] = this._query['include'] || [];
        this._query['include'] = this._query['include'].concat(val);
        return this;
    } else {
        console.error("Argument should be a String or an Array.");
    }
};

/**
 * @method locale
 * @description Set locale code, which locale's data wants to retrieve.
 * @param {String} locale_code - locale code. e.g. 'en-us', 'ja-jp', etc.
 * @returns {Entry}
 * @example .locale('en-us')
 */
Entry.prototype.locale = function (locale_code) {
    if (locale_code && typeof locale_code === 'string') {
        this._query['locale'] = locale_code;
        return this;
    } else {
        console.error("Argument should be a String.");
    }
};

/**
 * @method addQuery
 * @description add query to entity object.
 * @param {String} key - query key
 * @param {String} value - query value
 * @returns {Entry}
 * @example .addQuery('include_schema',true)
 */
Entry.prototype.addQuery = function (key, value) {
    if (key && value && typeof key === 'string') {
        this._query[key] = value;
        return this;
    } else {
        console.error("First argument should be a String.");
    }
};

/**
 * @method schema
 * @description Include schema in entries response.
 * @returns {Entry}
 * @example .include_schema()
 */
Entry.prototype.include_schema = function () {
    this._query['include_schema'] = true;
    return this;
};

/**
 * @method owner
 * @description Include owner in entries response.
 * @returns {Entry}
 * @example .include_owner()
 */
Entry.prototype.include_owner = function () {
    this._query['include_owner'] = true;
    return this;
};

/**
 * @method fetch
 * @description fetch entry of requested form of defined query if present.
 */
Entry.prototype.fetch = function () {
    if (this.entry_uid) {
        this.requestParams = {
            method: 'POST',
            headers: this.headers,
            url: this.config.protocol + "://" + this.config.host + '/' + this.config.version + '/forms/' + this.form_uid + '/entries/' + this.entry_uid,
            body: {
                _method: 'GET',
                query: this._query
            }
        };
        return this;
    } else {
        console.error("Kindly provide an entry uid. e.g. .Entry('bltsomething123')");
    }
};

/**
 * @method then
 * @description Execute query and get data from Built.io Contentstack
 *
 * @returns {function} returns success and fail functions with values respectively.
 */
Entry.prototype.then = function (success, fail) {
    var env_uid = this.environment_uid,
        params = this.requestParams;
    if (env_uid) {
        this._query.environment_uid = env_uid;
    } else {
        this._query.environment = this.environment;
    }
    Request(params, function (err, data) {
        try {
            if (!err) {
                if (this.singleEntry) {
                    var entries = {};
                    if (data.entries && data.entries.length) {
                        entries.entry = Utils.getData(data, this.headers.access_token).entries[0];
                        success(entries);
                    } else {
                        fail({ error_code: 141, error_message: 'The requested entry doesn\'t exist.' })
                    }
                    this.singleEntry = false;
                } else {
                    success(Utils.getData(data, this.headers.access_token));
                }
            } else {
                fail(err);
            }
        } catch (e) {
            fail({
                message: e.message
            });
        }
    }.bind(this));
};

module.exports = Entry;
