'use strict';
/**
 * Module dependencies.
 */
var Request = require('./../../lib/request');
var Utils = require('./../../lib/utils');
var Entry = require('./entry');

/**
 * Helper for expect and only
 * @api private
 */
var _extend = {
    compare: function (type) {
        return function (key, value) {
            if (key && value && typeof key === 'string' && typeof value === 'string') {
                this._query['query'][key] = this._query['query'][key] || {};
                this._query['query'][key][type] = value;
                return this;
            } else {
                console.error("Kindly provide valid parameters.");
            }
        };
    },
    contained: function (bool) {
        var type = (bool) ? '$in' : '$nin';
        return function (key, value) {
            if (key && value && typeof key === 'string' && Array.isArray(value)) {
                this._query['query'][key] = this._query['query'][key] || {};
                this._query['query'][key][type] = this._query['query'][key][type] || [];
                this._query['query'][key][type] = this._query['query'][key][type].concat(value);
                return this;
            } else {
                console.error("Kindly provide valid parameters.");
            }
        };
    },
    exists: function (bool) {
        return function (key) {
            if (key && typeof key === 'string') {
                this._query['query'][key] = this._query['query'][key] || {};
                this._query['query'][key]['$exists'] = bool;
                return this;
            } else {
                console.error("Kindly provide valid parameters.");
            }
        };
    },
    logical: function (type) {
        return function () {
            var _query = [];
            for (var i = 0, _i = arguments.length; i < _i; i++) {
                if (typeof arguments[i] === "object") {
                    _query.push(arguments[i]);
                }
            }
            if (this._query['query'][type]) {
                this._query['query'][type] = this._query['query'][type].concat(_query);
            } else {
                this._query['query'][type] = _query;
            }
            return this;
        };
    },
    sort: function (type) {
        return function (key) {
            if (key && typeof key === 'string') {
                this._query[type] = key;
                return this;
            } else {
                console.error("Argument should be a string.");
            }
        }
    },
    pagination: function (type) {
        return function (value) {
            if (value && typeof value === 'number') {
                this._query[type] = value;
                return this;
            } else {
                console.error("Argument should be a number.");
            }
        }
    }
};

function Query() {
    Entry.call(this);
    this._query = this._query || {};
    this._query['query'] = this._query['query'] || {};
}

/**
 * Inherit Entity prototype with Query
 */
Query.prototype = Object.create(Entry.prototype);

/**
 * @method getQuery
 * @description getQuery is get query object only for $and or $or
 * @returns {Query}
 * @example .where('title','Demo').getQuery()
 */
Query.prototype.getQuery = function () {
    return JSON.parse(JSON.stringify(this._query.query));
};

/**
 * @method where
 * @description Query to filter entries.
 * @param {String} key - find key in entry
 * @param {*} value - match or compare value in entry
 * @returns {Query}
 * @example .where('title','Demo')
 */
Query.prototype.where = function (key, value) {
    if (key && value && typeof key === 'string' && typeof value === 'string') {
        this._query['query'][key] = value;
        return this;
    } else {
        console.error("Kindly provide valid parameters.");
    }
};

/**
 * @method regex
 * @description Query to filter entries.
 * @param {String} key - find key in entry
 * @param {*} value - match or compare value in entry
 * @param {String} options - match or compare value in entry
 * @returns {Query}
 * @example .regex('title','/^Demo/')
 */
Query.prototype.regex = function (key, value, options) {
    if (key && value && typeof key === 'string' && typeof value === 'string') {
        this._query['query'][key] = {
            $regex: value
        };
        if (options) this._query['query'][key]['$options'] = options;
        return this;
    } else {
        console.error("Kindly provide valid parameters.");
    }
};

/**
 * @method search
 * @description Query to filter entries.
 * @param {string} value - search in entry
 * @returns {Query}
 * @example .search('Demo')
 */
Query.prototype.search = function (value) {
    if (value && typeof value === 'string') {
        this._query['typeahead'] = value;
        return this;
    } else {
        console.error("Kindly provide valid parameters.");
    }
};

/**
 * @method lessThan
 * @description Check less than.
 * @param {String} key - find key in entry
 * @param {*} value - match or compare value in entry
 * @returns {Query}
 * @example .lessThan('field_uid',123)
 */
Query.prototype.lessThan = _extend.compare('$lt');

/**
 * @method lessThanOrEqualTo
 * @description Check less than or equal to.
 * @param {String} key - find key in entry
 * @param {*} value - match or compare value in entry
 * @returns {Query}
 * @example .lessThanOrEqualTo('field_uid',123)
 */
Query.prototype.lessThanOrEqualTo = _extend.compare('$lte');

/**
 * @method greaterThan
 * @description Check greater than.
 * @param {String} key - find key in entry
 * @param {*} value - match or compare value in entry
 * @returns {Query}
 * @example .greaterThan('field_uid',123)
 */
Query.prototype.greaterThan = _extend.compare('$gt');

/**
 * @method greaterThanOrEqualTo
 * @description Check greater than or equal to.
 * @param {String} key - find key in entry
 * @param {*} value - match or compare value in entry
 * @returns {Query}
 * @example .greaterThanOrEqualTo('field_uid',123)
 */
Query.prototype.greaterThanOrEqualTo = _extend.compare('$gte');

/**
 * @method notEqualTo
 * @description Check not equal.
 * @param {String} key - find key in entry
 * @param {*} value - match or compare value in entry
 * @returns {Query}
 * @example .notEqualTo('title','Demo')
 */
Query.prototype.notEqualTo = _extend.compare('$ne');

/**
 * @method containedIn
 * @description Contained In.
 * @param {String} key - find key in entry
 * @param {*} value - match or compare value in entry
 * @returns {Query}
 * @example .containedIn('title','Demo')
 */
Query.prototype.containedIn = _extend.contained(true);

/**
 * @method notContainedIn
 * @description Not contained in.
 * @param {String} key - find key in entry
 * @param {Array} value - match or compare value in entry
 * @returns {Query}
 * @example .notContainedIn('title','Demo')
 */
Query.prototype.notContainedIn = _extend.contained(false);

/**
 * @method exists
 * @description A value is set for the key.
 * @param {String} key - find key in entry
 * @returns {Query}
 * @example .exists('title')
 */
Query.prototype.exists = _extend.exists(true);

/**
 * @method notExists
 * @description A value is set for the key.
 * @param {String} key - find key in entry
 * @returns {Query}
 * @example .notExists('title')
 */
Query.prototype.notExists = _extend.exists(false);

/**
 * @method or
 * @description A value is set for the key.
 * @param {String} key - find key in entry
 * @returns {Query}
 */
Query.prototype.or = _extend.logical('$or');

/**
 * @method and
 * @description A value is set for the key.
 * @param {String} key - find key in entry
 * @returns {Query}
 */
Query.prototype.and = _extend.logical('$and');

/**
 * @method query
 * @description A value is set for the key.
 * @param {String} query - find key in entry
 * @returns {Query}
 */
Query.prototype.query = function (query) {
    if (typeof query === "object") {
        this._query['query'] = Utils._merge(this._query['query'], query);
        return this;
    } else {
        console.error("Kindly provide valid parameters");
    }
};

/**
 * @method tags
 * @description The "tags" parameter allows you to specify an array o of tags to search objects.
 * @param {Array} values - tags
 * @returns {Query}
 */
Query.prototype.tags = function (values) {
    if (Array.isArray(values)) {
        this._query['tags'] = values;
        return this;
    } else {
        console.error("Kindly provide valid parameters");
    }
};

/**
 * @method ascending
 * @description The asc parameter will sort the returned objects in ascending order of the provided field uid.
 * @param {String} key - find key in entry
 * @returns {Query}
 * @example .ascending('title')
 */
Query.prototype.ascending = _extend.sort('asc');

/**
 * @method descending
 * @description The asc parameter will sort the returned objects in descending order of the provided field uid.
 * @param {String} key - find key in entry
 * @returns {Query}
 * @example .descending('title')
 */
Query.prototype.descending = _extend.sort('desc');

/**
 * @method include_count
 * @description Includes the count of the number of entries returned along with the actual entries in response
 * @returns {Query}
 * @example .include_count()
 */
Query.prototype.include_count = function () {
    this._query['include_count'] = true;
    return this;
};

/**
 * @method beforeUid
 * @description before uid.
 * @param {String} uid - UID.
 * @returns {Query}
 * @example .beforeUid('<entry uid>')
 */
Query.prototype.beforeUid = _extend.sort('before_uid');

/**
 * @method afterUid
 * @description after uid.
 * @param {String} uid - UID.
 * @returns {Query}
 * @example .afterUid('<entry uid>')
 */
Query.prototype.afterUid = _extend.sort('after_uid');

/**
 * @method skip
 * @description Skip number of entries from response.
 * @param {Number} skip - number of entries from response.
 * @returns {Query}
 * @example .skip(20)
 */
Query.prototype.skip = _extend.pagination('skip');

/**
 * @method limit
 * @description Set limit of response entries.
 * @param {Number} limit - number of entries in response.
 * @returns {Query}
 * @example .limit(20)
 */
Query.prototype.limit = _extend.pagination('limit');

/**
 * @method count
 * @description Gets only the count of the number of entries that pass the query condition
 * @returns {Query}
 */
Query.prototype.count = function () {
    this._query['count'] = true;
    this.requestParams = {
        method: 'POST',
        headers: this.headers,
        url: this.config.protocol + "://" + this.config.host + '/' + this.config.version + '/forms/' + this.form_uid + '/entries',
        body: {
            _method: 'GET',
            query: this._query
        }
    };
    return this;
};

/**
 * @method find
 * @description Finds entries of requested form of defined query if present.
 */
Query.prototype.find = function () {
    this.requestParams = {
        method: 'POST',
        headers: this.headers,
        url: this.config.protocol + "://" + this.config.host + '/' + this.config.version + '/forms/' + this.form_uid + '/entries',
        body: {
            _method: 'GET',
            query: this._query
        }
    };
    return this;
};

/**
 * @method findOne
 * @description Finds entries of requested form of defined query if present.
 */
Query.prototype.findOne = function () {
    this.singleEntry = true;
    this._query.limit = 1;
    this.requestParams = {
        method: 'POST',
        headers: this.headers,
        url: this.config.protocol + "://" + this.config.host + '/' + this.config.version + '/forms/' + this.form_uid + '/entries',
        body: {
            _method: 'GET',
            query: this._query
        }
    };
    return this;
};

module.exports = Query;
