'use strict';
/**
 * Module dependencies.
 */
var Request = require('./../lib/request');
var Utils = require('../lib/utils');
var siteCache = require('./site-cache');
var config = require('../config');
var Entry = require('./entry/entry');
var Query = require('./entry/query');

/**
 * Creates an instance of `Site`.
 *
 * An initializer is responsible for instantiating Built.io Contentstack.
 *
 * @constructor
 * @api protected
 */
function Site() {
    this.config = config;
}

/**
 * Initialize headers and environment.
 * @api protected
 */
Site.prototype.initialize = function () {
    var self = this;
    return function () {
        switch (arguments.length) {
            case 1:
                if (typeof arguments[0] === "object" && typeof arguments[0].site_api_key === "string" && typeof arguments[0].access_token === "string" && typeof arguments[0].environment === "string") {
                    self.headers = {
                        site_api_key: arguments[0].site_api_key,
                        access_token: arguments[0].access_token
                    };
                    self.environment = arguments[0].environment;
                    self._environment(arguments[0].environment, arguments[0].site_api_key);
                    return self;
                } else {
                    console.error("Kindly provide object parameters.");
                }
            case 3:
                if (typeof arguments[0] === "string" && typeof arguments[1] === "string" && typeof arguments[2] === "string") {
                    self.headers = {
                        site_api_key: arguments[0],
                        access_token: arguments[1]
                    };
                    self.environment = arguments[2];
                    self._environment(arguments[2], arguments[0]);
                    return self;
                } else {
                    console.error("Kindly provide string parameters.");
                }
            default:
                console.error("Kindly provide valid parameters to initialize the Built.io Contentstack Javascript SDK.");
        }
    };
};

/**
 * Get environment UID
 *
 * @param {String} env
 * @param {String} site_api_key
 * @api protected
 */
Site.prototype._environment = function (env, site_api_key) {
    var self = this,
        environmentCacheKey = site_api_key + '.environment.' + env;
    var cachedData = siteCache.get(environmentCacheKey);
    if (cachedData) {
        self.environment_uid = cachedData;
    } else {
        Request({
            url: self.config.protocol + "://" + self.config.host + '/' + self.config.version + '/environments/' + env,
            headers: self.headers
        }, function (err, data) {
            try {
                if (err) throw err;
                if (data && data.environment && data.environment.uid) {
                    siteCache.set(environmentCacheKey, data.environment.uid);
                    self.environment_uid = data.environment.uid;
                }
            } catch (e) {
                console.error('Could not retrieve the environment due to following error: ' + e.message);
            }
        });
    }
};

/**
 * @method Form
 * @description Site to filter Form.
 * @param {String} uid - form uid (optional)
 * @returns {Site}
 */
Site.prototype.Form = function (uid) {
    if (uid && typeof uid === 'string') {
        this.form_uid = uid;
    }
    return this;
};

/**
 * @method Entry
 * @description Set entry data.
 * @param {String} uid - entry uid
 * @returns {Site}
 */
Site.prototype.Entry = function (uid) {
    if (uid && typeof uid === "string") {
        this.entry_uid = uid;
    }
    this.__proto__.__proto__ = new Entry();
    return Object.create(this);
};

/**
 * @method Query
 * @description Site to query entries.
 * @returns {Site}
 */
Site.prototype.Query = function () {
    this.__proto__.__proto__ = new Query();
    return Object.create(this);
};

/**
 * Expose `Site`.
 */
module.exports = new Site();
